searchData={"items":[{"type":"module","title":"Peeper","doc":"`Peeper.GenServer` is an almost drop-in replacement for `GenServer` that preserves the state\n  between crashes. All the callbacks from `GenServer` are supported.\n\nInternally it creates a sub-supervision tree with an actual worker that delegates to the\n  implementation and a state keeper. That said, it creates three processes instead of one,\n  and this should be considered when building a very concurrent applications.\n\nThe main use-case would be a long-lived `GenServer` process with no error path handling at all\n  (yeah, that famous _Let It Crash_.) Using this abstraction, one is free to send unexpected\n  messages to the process, raise from its `handle_×××` clauses and whatnot.\n\nThere are two differencies compared to bare `GenServer`. `init/1` callback cannot return anything\n  but `{:ok, state}` tuple (this might have changed in the future,) and\n  `Peeper.{call/3,cast/2,send/2}` is to be used instead of `GenServer.{call/3,cast/2}` and\n  `Kernel.send/2` (this is not gonna change.)\n\nPlease note, that whatever is set in `init/1` callback as a state, will be overriden by\n  the latest state available upon restarts. That being said, `init/1` would play its role\n  in setting the state during the first run only. \n\nInstead of using `Peeper`’s counterparts, one might either name the process and use `Name.GenServer`\n  as a name of the underlying `GenServer` or get the `GenServer`’s `pid` via `Peeper.gen_server/1`\n  and use `GenServer.{call/3,cast/2}` with it.","ref":"Peeper.html"},{"type":"module","title":"Example - Peeper","doc":"iex> {:ok, pid} = Peeper.Impls.Full.start_link(state: 0, name: Counter)\n    ...> Peeper.call(pid, :state)\n    0\n    iex> Peeper.cast(pid, :inc)\n    :ok\n    iex> GenServer.call(Counter.GenServer, :state)\n    1\n    iex> Peeper.call(pid, :state)\n    1\n    iex> # emulate crash\n    ...> Process.exit(Peeper.Supervisor.worker(pid), :raise)\n    ...> %{} = Peeper.Supervisor.which_children(pid)\n    iex> Peeper.call(Counter, :state)\n    1\n    iex> Peeper.send(pid, :inc)\n    :inc\n    iex> Peeper.call(Counter, :state)\n    2\n\n### `start_link/1`\n\nThe function receives either an initial `state`, or a keyword having keys\n  `state` and (optionally) `name`. Also this keyword might have a configuration\n  for the top supervisor (keys: `[:strategy, :max_restarts, :max_seconds, :auto_shutdown]`.)\n\nAll other values passed would be re-passed to the underlying `GenServer`’s options.","ref":"Peeper.html#module-example"},{"type":"module","title":"Listener - Peeper","doc":"One might pass `listener: MyListener` key to `PeeperImpl.start_link/1` where `MyListener`\n  should be an implementation of `Peeper.Listener` behaviour. The callbacks will be called\n  when the state of the underlying behaviour is _changed_ and on termination respectively.\n\nThat might be a good place to attach telemetry events, or logs, or whatnot.","ref":"Peeper.html#module-listener"},{"type":"function","title":"Peeper.call/3","doc":"The counterpart for `GenServer.call/3`. Uses the very same semantics.","ref":"Peeper.html#call/3"},{"type":"function","title":"Peeper.cast/2","doc":"The counterpart for `GenServer.cast/2`. Uses the very same semantics.","ref":"Peeper.html#cast/2"},{"type":"function","title":"Peeper.gen_server/1","doc":"Returns the `pid` of the actual `GenServer`. Might be used to\navoid the necessity of calling other functions in this module.\n\nThe `pid` returned might be used directly in calls to\n`GenServer.{call/3,cast/2}` and/or `Kernel.send/2`","ref":"Peeper.html#gen_server/1"},{"type":"function","title":"Peeper.gen_server_name/1","doc":"Tries to produce a name for the underlying `GenServer` process.\nSucceeds if the name passed to `start_link/1` of the implementation\nis an `atom()`, `{:global, atom()}`, or `{:via, module(), atom()}`.\n\nIn such a case, the underlying `GenServer` module receives the name\n`Module.concat(name, GenServer)` and might be used as such.","ref":"Peeper.html#gen_server_name/1"},{"type":"function","title":"Peeper.send/2","doc":"The counterpart for `Kernel.send/2`. Uses the very same semantics.","ref":"Peeper.html#send/2"},{"type":"behaviour","title":"Peeper.GenServer","doc":"A drop-in replacement for `use GenServer`","ref":"Peeper.GenServer.html"},{"type":"macro","title":"Peeper.GenServer.__using__/1","doc":"Declares a `Peeper.GenServer` behaviour, injects `start_link/1` function\nand the child spec.","ref":"Peeper.GenServer.html#__using__/1"},{"type":"macro","title":"Example - Peeper.GenServer.__using__/1","doc":"```elixir\ndefmodule MyGenServer do\n  use Peeper.GenServer\n\n  @impl Peeper.GenServer\n  def handle_call(:state, _from, state),\n    do: {:reply, state, state}\n\n  @impl Peeper.GenServer\n  def handle_cast(:inc, state),\n    do: {:noreply, state, {:continue, :inc}}\n\n  @impl Peeper.GenServer\n  def handle_continue(:inc, state),\n    do: {:noreply, state + 1}\n  end\n```","ref":"Peeper.GenServer.html#__using__/1-example"},{"type":"callback","title":"Peeper.GenServer.code_change/3","doc":"_See:_ `c:GenServer.code_change/3`","ref":"Peeper.GenServer.html#c:code_change/3"},{"type":"callback","title":"Peeper.GenServer.handle_call/3","doc":"_See:_ `c:GenServer.handle_call/3`","ref":"Peeper.GenServer.html#c:handle_call/3"},{"type":"callback","title":"Peeper.GenServer.handle_cast/2","doc":"_See:_ `c:GenServer.handle_cast/2`","ref":"Peeper.GenServer.html#c:handle_cast/2"},{"type":"callback","title":"Peeper.GenServer.handle_continue/2","doc":"_See:_ `c:GenServer.handle_continue/2`","ref":"Peeper.GenServer.html#c:handle_continue/2"},{"type":"callback","title":"Peeper.GenServer.handle_info/2","doc":"_See:_ `c:GenServer.handle_info/2`","ref":"Peeper.GenServer.html#c:handle_info/2"},{"type":"callback","title":"Peeper.GenServer.init/1","doc":"Almost the same as `c:GenServer.init/1` but cannot return anything but `{:ok, new_state}`\n  tuple and is being invoked from `handle_continue/2` followed `init/1` of the state\n  keeper process.\n\n_See:_ `c:GenServer.init/1`","ref":"Peeper.GenServer.html#c:init/1"},{"type":"function","title":"Peeper.GenServer.start_link/2","doc":"Starts a `Peeper` sub-supervision process tree linked to the current process.","ref":"Peeper.GenServer.html#start_link/2"},{"type":"callback","title":"Peeper.GenServer.terminate/2","doc":"_See:_ `c:GenServer.terminate/2`","ref":"Peeper.GenServer.html#c:terminate/2"},{"type":"behaviour","title":"Peeper.Listener","doc":"The listener that might be implemented and attached to the `Peeper.GenServer`\n  to receive notifications about process state changes and/or terminations.","ref":"Peeper.Listener.html"},{"type":"callback","title":"Peeper.Listener.on_state_changed/2","doc":"Called when the underlying `Peeper.GenServer`’s state gets changed","ref":"Peeper.Listener.html#c:on_state_changed/2"},{"type":"callback","title":"Peeper.Listener.on_terminate/2","doc":"Called when the underlying `Peeper.GenServer` gets terminated","ref":"Peeper.Listener.html#c:on_terminate/2"},{"type":"extras","title":"Peeper","doc":"# Peeper\n\n**Almost drop-in replacement for `GenServer` to preserve state between crashes.**","ref":"readme.html"},{"type":"extras","title":"Benefits - Peeper","doc":"`Peeper.GenServer` is an almost drop-in replacement for `GenServer` that preserves the state\n  between crashes. All the callbacks from `GenServer` are supported.\n\nInternally it creates a sub-supervision tree with an actual worker that delegates to the\n  implementation and a state keeper. That said, it creates three processes instead of one,\n  and this should be considered when building a very concurrent applications.\n\nThe main use-case would be a long-lived `GenServer` process with no error path handling at all\n  (yeah, that famous _Let It Crash_.) Using this abstraction, one is free to send unexpected\n  messages to the process, raise from its `handle_×××` clauses and whatnot.\n\nThere are two differencies compared to bare `GenServer`. `init/1` callback cannot return anything\n  but `{:ok, state}` tuple (this might have changed in the future,) and\n  `Peeper.{call/3,cast/2,send/2}` is to be used instead of `GenServer.{call/3,cast/2}` and\n  `Kernel.send/2` (this is not gonna change.)\n\nPlease note, that whatever is set in `init/1` callback as a state, will be overriden by\n  the latest state available upon restarts. That being said, `init/1` would play its role\n  in setting the state during the first run only. \n\nInstead of using `Peeper`’s counterparts, one might either name the process and use `Name.GenServer`\n  as a name of the underlying `GenServer` or get the `GenServer`’s `pid` via `Peeper.gen_server/1`\n  and use `GenServer.{call/3,cast/2}` with it.","ref":"readme.html#benefits"},{"type":"extras","title":"Example - Peeper","doc":"```elixir\n    iex> {:ok, pid} = Peeper.Impls.Full.start_link(state: 0, name: Counter)\n    ...> Peeper.call(pid, :state)\n    0\n    iex> Peeper.cast(pid, :inc)\n    :ok\n    iex> GenServer.call(Counter.GenServer, :state)\n    1\n    iex> Peeper.call(pid, :state)\n    1\n    iex> # emulate crash\n    ...> Process.exit(Peeper.Supervisor.worker(pid), :raise)\n    ...> %{} = Peeper.Supervisor.which_children(pid)\n    iex> Peeper.call(Counter, :state)\n    1\n    iex> Peeper.send(pid, :inc)\n    :inc\n    iex> Peeper.call(Counter, :state)\n    2\n```\n\n### `start_link/1`\n\nThe function receives either an initial `state`, or a keyword having keys\n  `state` and (optionally) `name`. Also this keyword might have a configuration\n  for the top supervisor (keys: `[:strategy, :max_restarts, :max_seconds, :auto_shutdown]`.)\n\nAll other values passed would be re-passed to the underlying `GenServer`’s options.","ref":"readme.html#example"},{"type":"extras","title":"Listener - Peeper","doc":"One might pass `listener: MyListener` key to `PeeperImpl.start_link/1` where `MyListener`\n  should be an implementation of `Peeper.Listener` behaviour. The callbacks will be called\n  when the state of the underlying behaviour is _changed_ and on termination respectively.\n\nThat might be a good place to attach telemetry events, or logs, or whatnot.","ref":"readme.html#listener"},{"type":"extras","title":"Usage - Peeper","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Installation - Peeper","doc":"```elixir\ndef deps do\n  [\n    {:peeper, \"~> 0.1\"}\n  ]\nend\n```\n\n### [Documentation](https://hexdocs.pm/peeper)","ref":"readme.html#installation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}